local addon = LibStub("AceAddon-3.0"):NewAddon("InstanceLoot", "AceHook-3.0", "AceTimer-3.0")

local pt = LibStub("LibPeriodicTable-3.1")
local BB = LibStub("LibBabble-Boss-3.0")
local BBL = BB:GetLookupTable()
local BBR = BB:GetReverseLookupTable()
local BZ = LibStub("LibBabble-Zone-3.0")
local BZL = BZ:GetUnstrictLookupTable()
local BZR = BZ:GetReverseLookupTable()

local current_boss, current_zone, viewer, bosslist
local InstanceLoot_ScanTooltip

function addon:OnInitialize()
	InstanceLoot_ScanTooltip = CreateFrame("GameTooltip", "InstanceLoot_ScanTooltip", UIParent, "GameTooltipTemplate")
	InstanceLoot_ScanTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	
	self.bosses = {}
end

function addon:OnEnable()
	self:SecureHook("SetMapToCurrentZone")
	self:SecureHook(WorldMapFrame, "Hide", "HideLoot")
	self:SecureHook(WorldMapFrame, "Show", "SetMapToCurrentZone")
end

function addon:HasBosses(zone)
	if self.bosses[zone] ~= nil then return self.bosses[zone] ~= false end
	if pt("InstanceLoot."..zone) or pt("InstanceLootHeroic"..zone) then
		self:ExtractBossesFromPT("InstanceLoot", zone)
		self:ExtractBossesFromPT("InstanceLootHeroic", zone)
	else
		self.bosses[zone] = false
	end
	if self.bosses[zone] then return true end
end

function addon:ExtractBossesFromPT(base, zone)
	local sets = pt(base .. '.' .. zone)
	if not sets then return end
	local pattern = "^"..base.."%."..zone.."%.([^.]+)$"
	for _,set in pairs(sets) do
		local boss = set.set:match(pattern)
		if boss then
			if not self.bosses[zone] then self.bosses[zone] = {} end
			self.bosses[zone][boss] = true
		end
	end
end

function addon:SetMapToCurrentZone()
	if not WorldMapFrame:IsVisible() then return end
	self:ShowBossList(BZR[GetRealZoneText()])
end

function addon:HideLoot()
	self:HideBoss()
	self:HideBossList()
end
function addon:HideBoss()
	if viewer then
		current_boss = nil
		viewer:Hide()
	end
end
function addon:HideBossList()
	if bosslist then
		current_zone = nil
		bosslist:Hide()
	end
end

function addon:ShowBossList(zone)
	current_zone = zone

	self:UpdateBossList()
end

function addon:ShowBoss(boss)
	if not current_zone then return end
	boss = BBR[boss] and BBR[boss] or boss
	if boss == current_boss then
		return addon:HideBoss()
	end
	current_boss = boss

	self:UpdateViewer()
end

local bosses = {}
function addon:UpdateBossList()
	if not current_zone then return end
	if not self:HasBosses(current_zone) then return end

	if not bosslist then
		bosslist = LibStub("LibSimpleFrame-1.0"):New("InstanceLoot_BossList", {
			position = {point="RIGHT", relpoint="CENTER", x=0, y=0},
			--position = {point="CENTER", x=0, y=0},
			lock = false,
			scale = 1,
			strata = "FULLSCREEN",
			fade = 1,
			opacity = 1,
			width = 250,
			border = {0.9, 0.82, 0, 1},
			background = {0, 0, 0, 1},
			min_height = 20,
		})
		local close = CreateFrame("Button", nil, bosslist, "UIPanelCloseButton")
		close:SetPoint("TOPRIGHT", bosslist)
		close:SetScript("OnClick", self.HideBossList)
	end
	
	-- do something to work out LBRS/UBRS here, I suppose.

	table.wipe(bosses)
	for boss in pairs(self.bosses[current_zone]) do
		table.insert(bosses, boss)
	end
	table.sort(bosses)

	bosslist:Clear()

	bosslist:AddLine(BZL[current_zone] and BZL[current_zone] or current_zone):Color(1, 1, 0)
	bosslist:AddLine()
	for _, boss in pairs(bosses) do
		bosslist:AddLine(boss):Handler("OnLeftClick", self.BossClick, boss):Color(0, 0.5, 1)
	end
	
	bosslist:Size()
	bosslist:SetPosition()
	bosslist:Show()
end

function addon.BossClick(_, boss)
	addon:ShowBoss(boss)
end

function addon:UpdateViewer()
	if not (current_boss and current_zone) then return end
	if not viewer then
		viewer = LibStub("LibSimpleFrame-1.0"):New("InstanceLoot_Viewer", {
			position = {point="LEFT", relpoint="CENTER", x=0, y=0},
			--position = {point="CENTER", x=0, y=0},
			lock = false,
			scale = 1,
			strata = "FULLSCREEN_DIALOG",
			fade = 1,
			opacity = 1,
			width = 250,
			border = {0.9, 0.82, 0, 1},
			background = {0, 0, 0, 1},
			min_height = 20,
		})
		local close = CreateFrame("Button", nil, viewer, "UIPanelCloseButton")
		close:SetPoint("TOPRIGHT", viewer)
		close:SetScript("OnClick", self.HideBoss)
	end

	-- haxxor
	if (current_zone == "Blackrock Spire") then
		if pt("InstanceLoot.Upper " .. current_zone .. "." .. current_boss) then
			current_zone = "Upper Blackrock Spire"
		elseif pt("InstanceLoot.Lower " .. current_zone .. "." .. current_boss) then
			current_zone = "Lower Blackrock Spire"
		end
	end
	-- end haxxor

	if not pt("InstanceLoot." .. current_zone .. "." .. current_boss) and not pt("InstanceLootHeroic." .. current_zone .. "." .. current_boss) then
		self:Print(("PT3 has no table InstanceLoot.%s.%s"):format(current_zone, current_boss))
		return self:HideBoss()
	end
	
	viewer:Clear()
	
	viewer:AddLine(BBL[current_boss])

	if pt("InstanceLoot." .. current_zone .. "." .. current_boss) then
		viewer:AddLine()
		viewer:AddLine("Normal Drops")
		self:AddSetTo(viewer, "InstanceLoot." .. current_zone .. "." .. current_boss)
	end
	if pt("InstanceLootHeroic." .. current_zone .. "." .. current_boss) then
		viewer:AddLine()
		viewer:AddLine("Heroic Drops")
		self:AddSetTo(viewer, "InstanceLootHeroic." .. current_zone .. "." .. current_boss)
	end

	--viewer:AddLine("Close"):Handler("OnLeftClick", hide_viewer)

	viewer:Size()
	viewer:SetPosition()
	viewer:Show()
end

local sort_set
do
	local itemids, droprates = {}, {}
	local function by_droprate(a, b)
		return tonumber(droprates[a]) > tonumber(droprates[b])
	end
	function sort_set(set)
		table.wipe(itemids)
		table.wipe(droprates)
		for id, drop in pt:IterateSet(set) do
			droprates[id] = drop
			table.insert(itemids, id)
		end
		table.sort(itemids, by_droprate)
		return itemids, droprates
	end
end

function addon:AddSetTo(frame, set)
	local itemids, droprates = sort_set(set)
	for i, id in ipairs(itemids) do
		local name, link, rarity, _,_,_,_,_,_, texture = GetItemInfo(id)
		local percentage = ("%.1f%%"):format(droprates[id] / 10)
		local line
		if link then
			local r, g, b = GetItemQualityColor(rarity)
			line = frame:AddLine("|T"..texture..":12|t "..name, percentage):Color(r, g, b)
				:Handler("OnEnter", self.OnMouseEnter, link)
		else
			line = frame:AddLine(id, percentage):Color(1, 0, 0)
				:Handler("OnEnter", self.OnUnknownEnter)
		end
		line:Handler("OnLeftClick", self.DropClick, id):Handler("OnLeave", self.OnMouseLeave)
	end
end

function addon.DropClick(_, id)
	local name, link, rarity, _,_,_,_,_,_, texture = GetItemInfo(id)
	if not name then
		InstanceLoot_ScanTooltip:SetOwner(UIParent, "ANCHOR_NONE")
		InstanceLoot_ScanTooltip:SetHyperlink("item:" .. id)
		InstanceLoot_ScanTooltip:Hide()
		addon:ScheduleTimer(addon.UpdateViewer, 0.5, addon)
		return
	end
	if IsModifiedClick("DRESSUP") then
		DressUpItemLink(link)
	elseif IsModifiedClick("CHATLINK") then
		if not ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Show()
		end
		ChatFrameEditBox:Insert(link)
	else
		ShowUIPanel(ItemRefTooltip)
		if not ItemRefTooltip:IsVisible() then
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
		end
		ItemRefTooltip:SetHyperlink(link)
	end
end

function addon.OnMouseEnter(_, link)
	GameTooltip:SetOwner(viewer, "ANCHOR_RIGHT")
	GameTooltip:SetHyperlink(link)
end

function addon.OnUnknownEnter()
	GameTooltip:SetOwner(viewer, "ANCHOR_RIGHT")
	GameTooltip:SetText("Unknown item; click to query")
end

function addon.OnMouseLeave()
	GameTooltip:Hide()
end

