local addon = LibStub("AceAddon-3.0"):GetAddon("InstanceLoot")
local mod = addon:NewModule("Browser", "AceConsole-3.0")

local pt = LibStub("LibPeriodicTable-3.1")
local BZ = LibStub("LibBabble-Zone-3.0")
local BZL = BZ:GetUnstrictLookupTable()

function mod:OnInitialize()
	self:RegisterChatCommand("loot", "ShowInstanceList", false)
end

local instances = {}
function mod:GenerateInstanceList()
	for zone in BZ:Iterate() do
		addon:HasBosses(zone)
	end
	addon:HasBosses("World Bosses") -- hax

	for zone, bosses in pairs(addon.bosses) do
		if bosses then
			table.insert(instances, zone)
		end
	end
	table.sort(instances)
end

local instancelist, slider
local offset, limit = 0, 25
function mod:ShowInstanceList()
	if not instancelist then
		self:GenerateInstanceList()
		instancelist = LibStub("LibSimpleFrame-1.0"):New("InstanceLoot_InstanceList", {
			position = {point="LEFT", relpoint="LEFT", x=0, y=0},
			--position = {point="CENTER", x=0, y=0},
			lock = false,
			scale = 1,
			strata = "DIALOG",
			fade = 1,
			opacity = 1,
			width = 250,
			border = {0.9, 0.82, 0, 1},
			background = {0, 0, 0, 1},
			min_height = 20,
			max_height = 100,
		})
		local close = CreateFrame("Button", nil, instancelist, "UIPanelCloseButton")
		close:SetPoint("TOPRIGHT", instancelist)
		close:SetScript("OnClick", function()
			instancelist:Hide()
		end)
		slider = CreateFrame("Slider", "InstanceLoot_InstanceListSlider", instancelist, "OptionsSliderTemplate")
		_G["InstanceLoot_InstanceListSliderText"]:Hide()
		_G["InstanceLoot_InstanceListSliderLow"]:Hide()
		_G["InstanceLoot_InstanceListSliderHigh"]:Hide()
		slider:SetMinMaxValues(0, #instances - limit)
		slider:SetValue(0)
		slider:SetOrientation("VERTICAL")
		slider:SetWidth(20)
		slider:SetPoint("TOPLEFT", instancelist, "TOPRIGHT")
		slider:SetScript("OnValueChanged", function(self)
			offset = slider:GetValue()
			mod:ShowInstanceList()
		end)
		slider:Show()
	end

	instancelist:Clear()

	instancelist:AddLine("Instances")
	instancelist:AddLine()
	local n = 1
	for i, zone in ipairs(instances) do
		if i > offset and n < limit then
			instancelist:AddLine(BZL[zone] and BZL[zone] or zone):Color(0, 1, 0):Handler("OnLeftClick", self.InstanceClick, zone)
			n = n + 1
		end
	end
	
	instancelist:Size()
	instancelist:SetPosition()

	slider:SetHeight(instancelist:GetHeight())

	instancelist:Show()
end

function mod.InstanceClick(_, zone)
	addon:ShowBossList(zone)
end

